package org.ow2.odis.core;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import org.jdom.Element;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.connection.ConnectionFactory;
import org.ow2.odis.connection.socket.OdisProtocol;
import org.ow2.odis.loader.GenericFactory;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.model.AbstractAttribute;
import org.ow2.odis.model.AbstractConnectionAttribute;
import org.ow2.odis.model.Const;
import org.ow2.odis.sql.IProtocolIn;
import org.ow2.odis.sql.IProtocolOut;
import org.ow2.odis.util.CheckUtilities;

/**
 * The Class AConnectionAttribute.
 */
public abstract class AConnectionAttribute extends AbstractConnectionAttribute {
    /** The Constant PROPERTY_SERVER_SIDE. */
    public static final String PROPERTY_SERVER_SIDE = "SERVER_SIDE";

    /** The Constant PROPERTY_PROTOCOL_IN. */
    static final String PROPERTY_PROTOCOL_IN = "PROTOCOL_IN";

    /** The Constant PROPERTY_PROTOCOL_OUT. */
    static final String PROPERTY_PROTOCOL_OUT = "PROTOCOL_OUT";

    /** The Constant PROPERTY_PROPERTIES. */
    static final String PROPERTY_PROPERTIES = "PROPERTIES";

    /** The Constant PROPERTY_IN. */
    public static final String PROPERTY_IN = "IN";

    /** The Constant PROPERTY_OUT. */
    public static final String PROPERTY_OUT = "OUT";

    /** The Constant PROPERTY_TIMEOUT. */
    public static final String PROPERTY_TIMEOUT = "TIMEOUT";

    /** The Constant PROPERTY_SIZE_BYTE_MAX. */
    private static final String PROPERTY_SIZE_BYTE_MAX = "SIZE_BYTE_MAX";

    /** The Constant PROPERTY_SIZE_MAX_OBJECT. */
    private static final String PROPERTY_SIZE_MAX_OBJECT = "SIZE_MAX_OBJET";

    /** The size max object. */
    private int sizeMaxObject = 5000;

    /** The str size max object. */
    private String strSizeMaxObject = "5000";

    /** The size byte max. */
    private int sizeByteMax = 100000;

    /** The str size byte max. */
    private String strSizeByteMax = "100000";

    /** The protocol in. */
    private String protocolIn = OdisProtocol.class.getName();

    /** The default protocol out. */
    private String defaultProtocolOut = OdisProtocol.class.getName();

    /** The default protocol in. */
    private String defaultProtocolIn = OdisProtocol.class.getName();

    /** The protocol out. */
    private String protocolOut = OdisProtocol.class.getName();

    /** The protocol in version. */
    protected String protocolInVersion = GenericFactory.DEFAULT_VERSION;

    /** The protocol out version. */
    protected String protocolOutVersion = GenericFactory.DEFAULT_VERSION;

    /** The server side. */
    private String serverSide = "IN";

    /** The state. */
    private int state = Const.READY;

    /** The size byte. */
    private long sizeByte = 0;

    /** The protocol in properties. */
    private String protocolInProperties = "";

    /** The protocol out properties. */
    private String protocolOutProperties = "";

    /** The time out. */
    private int timeOut = 0;

    /** The str time out. */
    private String strTimeOut = "0";

    /** The min client. */
    private int minClient = 0;

    /** The str min client. */
    private String strMinClient = "0";

    /**
     * Constructor.
     * @param parent AbstractAttribut
     */
    public AConnectionAttribute(AbstractAttribute parent) {
        super(parent);
        setDefaultProtocolIn();
        setDefaultProtocolOut();
        this.protocolIn = this.defaultProtocolIn;
        this.protocolOut = this.defaultProtocolOut;
    }

    /**
     * set default protocol IN.
     */
    protected abstract void setDefaultProtocolIn();

    /**
     * set default protocol IOUT.
     */
    protected abstract void setDefaultProtocolOut();

    /**
     * equals.
     * @param obj to be compare
     * @return boolean
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        boolean result = false;
        AConnectionAttribute copy = null;
        try {
            copy = (AConnectionAttribute) obj;
        } catch (ClassCastException e) {
            return false;
        }
        result = super.equals(obj);
        if (result) {
            result &= ((protocolIn == null && copy.protocolIn == null) || copy.protocolIn
                    .equals(this.protocolIn));
            result &= ((protocolInProperties == null && copy.protocolInProperties == null) || copy.protocolInProperties
                    .equals(this.protocolInProperties));
            result &= ((protocolOut == null && copy.protocolOut == null) || copy.protocolOut
                    .equals(this.protocolOut));
            result &= ((protocolOutProperties == null && copy.protocolOutProperties == null) || copy.protocolOutProperties
                    .equals(this.protocolOutProperties));
            result &= ((serverSide == null && copy.serverSide == null) || copy.serverSide
                    .equals(this.serverSide));
            result &= ((strMinClient == null && copy.strMinClient == null) || copy.strMinClient
                    .equals(this.strMinClient));
            result &= ((strSizeByteMax == null && copy.strSizeByteMax == null) || copy.strSizeByteMax
                    .equals(this.strSizeByteMax));
            result &= ((strSizeMaxObject == null && copy.strSizeMaxObject == null) || copy.strSizeMaxObject
                    .equals(this.strSizeMaxObject));
            result &= ((strTimeOut == null && copy.strTimeOut == null) || copy.strTimeOut
                    .equals(this.strTimeOut));
            if (!result) {
                traceOnConfictualConnection(copy);
            }
        }
        return result;
    }

    /**
     * getter.
     * @return the size byte contain into socket connection
     */
    public long getSizeByte() {
        return sizeByte;
    }

    /**
     * increase (or decrease in case of negative param) the size of byte contain into the socket connexion.
     * @param value value of increament. It can be negative.
     */
    public void incSizeByte(long value) {
        sizeByte += value;
    }

    /** The number message. */
    private int numberMessage = 0;

    /**
     * increase (or decrease in case of negative param) the number of message contain by the socket connexion.
     * @param value value of increament. It can be negative.
     */
    public void incNumberMessage(int value) {
        numberMessage += value;
    }

    /** The number connexion. */
    private int numberConnexion = 0;

    /**
     * increase (or decrease in case of negative param) the number connexion contain by the socket connexion. This number represent both
     * client and server connexion.
     * @param value value of increament. It can be negative.
     */
    public void incNumberConnexion(int value) {
        numberConnexion += value;
    }

    /**
     * Setter.
     * @param cnxDeclaration the declaration
     */
    public void subSetter(Element cnxDeclaration) {
        List lsUnusedProperties = cnxDeclaration.getChildren();
        Element element = cnxDeclaration.getChild(PROPERTY_SERVER_SIDE);
        if (element != null) {
            serverSide = element.getAttributeValue(Const.VALUE, serverSide);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(PROPERTY_TIMEOUT);
        if (element != null) {
            strTimeOut = element.getAttributeValue(Const.VALUE, strTimeOut);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(PROPERTY_SIZE_BYTE_MAX);
        if (element != null) {
            strSizeByteMax = element.getAttributeValue(Const.VALUE,
                    strSizeByteMax);
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(PROPERTY_SIZE_MAX_OBJECT);
        if (element != null) {
            strSizeMaxObject = element.getAttributeValue(Const.VALUE,
                    strSizeMaxObject);
            lsUnusedProperties.remove(element);
        }
        subSubSetter(cnxDeclaration, lsUnusedProperties);
        CheckUtilities.checkUnusedProperties(lsUnusedProperties, LOGGER);
    }

    /**
     * Sets the values for implementation dependent elements.<br>
     * For a standard SOCKET connection, these are ProtocolIn and ProtocolOut.
     * @param cnxDeclaration the connection declaration
     * @param lsUnusedProperties the list of unused properties, managed by subSetter
     */
    public void subSubSetter(Element cnxDeclaration, List lsUnusedProperties) {
        Element element = cnxDeclaration.getChild(PROPERTY_PROTOCOL_IN);
        if (element != null) {
            protocolIn = element.getAttributeValue(Const.VALUE, protocolIn);
            Element subElement = (Element) element
                    .getChild(PROPERTY_PROPERTIES);
            if (subElement != null) {
                protocolInProperties = subElement.getAttributeValue(
                        Const.VALUE, protocolInProperties);
            }
            lsUnusedProperties.remove(element);
        }
        element = cnxDeclaration.getChild(PROPERTY_PROTOCOL_OUT);
        if (element != null) {
            protocolOut = element.getAttributeValue(Const.VALUE, protocolOut);
            Element subElement = (Element) element
                    .getChild(PROPERTY_PROPERTIES);
            if (subElement != null) {
                protocolOutProperties = subElement.getAttributeValue(
                        Const.VALUE, protocolOutProperties);
            }
            lsUnusedProperties.remove(element);
        }
    }

    /**
     * getter.
     * @return the type of this connection.
     */
    public String getTypeCnx() {
        return ConnectionFactory.CONNECTION_SOCKET;
    }

    /**
     * create a new ProtocoleIn according to the class name.
     * @return new IProtocoleIn instance
     * @throws OdisAttributException can occurred if the class name is unknown or if this class doesn't inherit from IProtocoleIn
     */
    public IProtocolIn createProtocolInInstance() throws OdisAttributeException {
        IProtocolIn result = null;
        try {
            result = (IProtocolIn) CheckUtilities.createInstance(protocolIn,
                    protocolInVersion, "protocolIn", LOGGER);
            result.setProperties(this.protocolInProperties);
            result.setConnectionAttribute(this);
        } catch (ClassCastException e) {
            StringBuffer buff = new StringBuffer("protocolIn class \"");
            buff.append(protocolIn);
            buff.append("\" is invalid");
            LOGGER.log(BasicLevel.ERROR, buff.toString());
            throw new OdisAttributeException(buff.toString(), e);
        } catch (Exception e) {
            throw new OdisAttributeException(e);
        }
        return result;
    }

    /**
     * create a new ProtocoleOut according to the class name.
     * @return new IProtocoleOut instance
     * @throws OdisAttributException can occurred if the class name is unknown or if this class doesn't inherit from IProtocoleOut
     */
    public IProtocolOut createProtocolOutInstance()
            throws OdisAttributeException {
        IProtocolOut result = null;
        try {
            result = (IProtocolOut) CheckUtilities.createInstance(protocolOut,
                    protocolOutVersion, "protocolOut", LOGGER);
            result.setProperties(this.protocolOutProperties);
            result.setConnectionAttribute(this);
        } catch (ClassCastException e) {
            StringBuffer buff = new StringBuffer("protocolOut class \"");
            buff.append(protocolOut);
            buff.append("\" is invalid");
            LOGGER.log(BasicLevel.ERROR, buff.toString());
            throw new OdisAttributeException(buff.toString(), e);
        } catch (Exception e) {
            throw new OdisAttributeException(e);
        }
        return result;
    }

    /**
     * Checks the validity of all AbstractCnxAttribut parameters.
     * @throws OdisAttributException exception in case of error
     * @see com.francetelecom.odis.model.AbstractAttribut#checkProperties()
     */
    public void checkProperties() throws OdisAttributeException {
        StringBuffer buff = new StringBuffer();
        try {
            setTimeOut(Integer.parseInt(strTimeOut));
        } catch (NumberFormatException e) {
            buff = concatException(buff, PROPERTY_TIMEOUT, strTimeOut, e);
        }
        try {
            setSizeByteMax(Integer.parseInt(strSizeByteMax));
        } catch (NumberFormatException e) {
            buff = concatException(buff, Const.PROPERTY_SIZE_BYTE_MAX,
                    strSizeByteMax, e);
        }
        try {
            setSizeObjectMax(Integer.parseInt(strSizeMaxObject));
        } catch (NumberFormatException e) {
            buff = concatException(buff, Const.PROPERTY_SIZE_MAX_OBJECT,
                    strSizeMaxObject, e);
        }
        if (!(serverSide.equals("IN") || serverSide.equals("OUT"))) {
            buff = concatException(buff, Const.PROPERTY_SERVER_SIDE,
                    serverSide, null);
        }
        checkProperties(buff);
        if (!"".equals(buff.toString())) {
            LOGGER.log(BasicLevel.ERROR, buff.toString());
            throw new OdisAttributeException(buff.toString());
        }
    }

    /**
     * Check the properties of the extended ACnxAttribut.
     * @param buff the buff
     * @return the string
     */
    protected abstract void checkProperties(StringBuffer buff);

    /**
     * Checks the validity of implementation specific parameters.<br>
     * For a standard SOCKET connection, these are ProtocolIn and ProtocolOut.
     * @param buff a StringBuffer used to store the problems found.
     */
    public void subCheckProperties(StringBuffer buff) {
        try {
            createProtocolInInstance();
        } catch (OdisAttributeException e) {
            buff = concatException(buff, PROPERTY_PROTOCOL_IN, protocolIn, e);
        }
        try {
            createProtocolOutInstance();
        } catch (OdisAttributeException e) {
            buff = concatException(buff, PROPERTY_PROTOCOL_OUT, protocolOut, e);
        }
    }

    /**
     * getter.
     * @return String class name of protocoleIn
     */
    public String getProtocolIn() {
        return protocolIn;
    }

    /**
     * setter.
     * @param pProtocolIn class name of protocoleIn
     */
    public void setProtocolIn(String pProtocolIn) {
        this.protocolIn = pProtocolIn;
    }

    /**
     * getter.
     * @return the class name use for the protocol Out
     */
    public String getProtocolOut() {
        return protocolOut;
    }

    /**
     * setter.
     * @param pProtocolOut the class name use for the protocol Out
     */
    public void setProtocolOut(String pProtocolOut) {
        this.protocolOut = pProtocolOut;
    }

    /**
     * setter of server side witch determine where server is located.
     * @param side can be IN or OUT value
     */
    public void setServerSide(String side) {
        if (PROPERTY_IN.equals(side)) {
            serverSide = PROPERTY_IN;
        } else if (PROPERTY_OUT.equals(side)) {
            serverSide = PROPERTY_OUT;
        } else {
            LOGGER.log(BasicLevel.ERROR, "unknown value " + side + " for "
                    + PROPERTY_SERVER_SIDE + " 's property");
        }
        this.serverSide = side;
    }

    /**
     * getter of server side.
     * @return the server location.
     */
    public String getServerSide() {
        return serverSide;
    }

    /**
     * getter.
     * @return the size max of object contain by SocketCnxAttribut
     */
    public int getSizeObjectMax() {
        return sizeMaxObject;
    }

    /**
     * setter.
     * @param sizeMax the size max of object contain by SocketCnxAttribut
     */
    public void setSizeObjectMax(int sizeMax) {
        this.sizeMaxObject = sizeMax;
    }

    /**
     * getter.
     * @return the size max of bytes contain by SocketCnxAttribut
     */
    public int getSizeByteMax() {
        return sizeByteMax;
    }

    /**
     * setter.
     * @param pValue the size max of bytes contain by SocketCnxAttribut
     */
    public void setSizeByteMax(int pValue) {
        this.sizeByteMax = pValue;
        setChanged();
        this.notifyObservers(this);
    }

    /**
     * Gets the state.
     * @return the state
     */
    public int getState() {
        return state;
    }

    /**
     * Sets the state. *
     * @param pState the new state
     */
    public void setState(int pState) {
        this.state = pState;
    }

    /**
     * getter.
     * @return string representing the state.
     */
    public String getStateString() {
        return state == Const.READY ? Const.STR_READY : Const.STR_NOT_READY;
    }

    /**
     * getter.
     * @return the number of message contained by the connection.
     */
    public int getNumberMessage() {
        return numberMessage;
    }

    /**
     * getter.
     * @return the number of connection done on a socket
     * @see com.francetelecom.odis.model.SocketCnxAttributMBean#getNumberConnexion()
     */
    public int getNumberConnexion() {
        return numberConnexion;
    }

    /**
     * getter.
     * @return the time out before the connection break if no response
     */
    public int getTimeOut() {
        return timeOut;
    }

    /**
     * setter.
     * @param pTimeOut the time out before the connection break if no response
     */
    public void setTimeOut(int pTimeOut) {
        this.timeOut = pTimeOut;
    }

    /**
     * Write XML.
     * @param out the out
     * @param full the full
     * @throws java.io.IOException *
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public void writeXML(OutputStream out, boolean full) throws IOException {
        writeXMLCNXUtilities(out, 2, PROPERTY_SERVER_SIDE, getServerSide(), "",
                full);
        if ((protocolInProperties != null)
                && (!"".equals(protocolInProperties))) {
            writeXMLCNXUtilities(out, 2, PROPERTY_PROTOCOL_IN, false, "value",
                    protocolIn);
            writeXMLCNXUtilities(out, 3, PROPERTY_PROPERTIES,
                    protocolInProperties, "", full);
            writeXMLCNXUtilities(out, 2, PROPERTY_PROTOCOL_IN, true);
        } else {
            writeXMLCNXUtilities(out, 2, PROPERTY_PROTOCOL_IN, getProtocolIn(),
                    this.defaultProtocolIn, full);
        }
        if ((protocolOutProperties != null)
                && (!"".equals(protocolOutProperties))) {
            writeXMLCNXUtilities(out, 2, PROPERTY_PROTOCOL_OUT, false, "value",
                    protocolOut);
            writeXMLCNXUtilities(out, 3, PROPERTY_PROPERTIES,
                    protocolOutProperties, "", full);
            writeXMLCNXUtilities(out, 2, PROPERTY_PROTOCOL_OUT, true);
        } else {
            writeXMLCNXUtilities(out, 2, PROPERTY_PROTOCOL_OUT,
                    getProtocolOut(), this.defaultProtocolOut, full);
        }
    }

    /**
     * Sets the default protocol out.
     * @param defaultProtocolOut the default protocol out
     */
    public void setDefaultProtocolOut(String defaultProtocolOut) {
        this.defaultProtocolOut = defaultProtocolOut;
    }

    /**
     * Sets the default protocol in.
     * @param defaultProtocolIn the default protocol in
     */
    public void setDefaultProtocolIn(String defaultProtocolIn) {
        this.defaultProtocolIn = defaultProtocolIn;
    }

    /**
     * Gets the protocol IN properties
     * @return String
     */
    public String getProtocolInProperties() {
        return protocolInProperties;
    }

    /**
     * Sets the Protocol IN properties
     * @param protocolInProperties
     */
    public void setProtocolInProperties(String protocolInProperties) {
        this.protocolInProperties = protocolInProperties;
    }

    /**
     * Gets the protocol OUT properties
     * @return String
     */
    public String getProtocolOutProperties() {
        return protocolOutProperties;
    }

    /**
     * Sets the protocol OUT properties
     * @param protocolOutProperties
     */
    public void setProtocolOutProperties(String protocolOutProperties) {
        this.protocolOutProperties = protocolOutProperties;
    }
}
