package org.ow2.odis.core;

import org.ow2.odis.connection.context.ContextMessage;
import org.ow2.odis.connection.context.IContextualConnectionOut;
import org.ow2.odis.port.PortOut;

/**
 * The Class AbstractTCPConnexionOut.
 */
public abstract class AbstractConnectionOut extends AbstractConnection
        implements IContextualConnectionOut {
    /** The port out. */
    protected PortOut portOut = null;

    /**
     * Instantiates a new abstract connection out.
     * @param attribut the attribute
     */
    public AbstractConnectionOut(AConnectionAttribute attribut) {
        super(attribut);
    }

    /**
     * try to send message to the client.
     * @param pMsg byte[] message
     * @return true if succeed false in other case
     */
    public final boolean send(byte[] pMsg) {
        boolean result = false;
        if (super.getMessageSender() != null) {
            result = super.getMessageSender().send(pMsg);
        }
        return result;
    }

    /**
     * try to send contextual message to the client.
     * @param pMsg ContextMessage message
     * @return true if succeed false in other case
     */
    public final boolean send(ContextMessage pMsg) {
        boolean result = false;
        if (super.getMessageSender() != null) {
            result = super.getMessageSender().send(pMsg);
        }
        return result;
    }

    /**
     * @see com.francetelecom.odis.component.lifeCycle.connexion.ICnxLifeCycle#unloadComponent()
     */
    public void unloadComponent() {
        attribute.deleteObserver(this);
    }
}
