package org.ow2.odis.core;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.context.Context;
import org.ow2.odis.lifeCycle.connection.IConnectionLifeCycle;


/**
 * Listener or sender : process the message read from or send to the serial port.
 * @author lqzp4386
 */
public abstract class MessageProcessor {
    /** keep a connection reference. */
    IConnection cnx = null;

    /** logger of this class/. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            MessageProcessor.class.getName());

    /** if an error occurred stopProcessing is fixed to true. */
    public boolean stopProcessing = false;

    /** keep a server reference. */
    protected IConnectionLifeCycle server = null;

    /** The mode. */
    protected String mode = null;

    /** The context. */
    protected Context context;

    /**
     * the protocol instance witch will be used for writing a message on this serial port.
     */
    protected IProtocol protocol = null;

    /**
     * Instantiates a new message processor.
     * @param server the server
     * @param mode the mode
     * @param serialPort the serial port
     */
    public MessageProcessor(IConnectionLifeCycle server, IConnection cnx) {
        this.server = server;
        this.cnx = cnx;
    }

    /**
     * Gets the serial port.
     * @return SerialPort.
     */
    public IConnection getConnection() {
        return cnx;
    }

    /**
     * Sets the connexion.
     * @param cnx IConnection
     */
    public void setConnection(IConnection cnx) {
        this.cnx = cnx;
        this.context = cnx.getContext();
    }

    /**
     * Start.
     */
    protected abstract void start();

    /**
     * Gets the protocol.
     * @return the protocol
     */
    public IProtocol getProtocol() {
        return protocol;
    }

    /**
     * Sets the protocol.
     * @param protocol the new protocol
     */
    public void setProtocol(IProtocol protocol) {
        this.protocol = protocol;
    }
}
