package org.ow2.odis.sql;

import java.util.Iterator;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.core.AbstractConnectionIn;
import org.ow2.odis.core.IConnection;
import org.ow2.odis.core.IMessageListener;
import org.ow2.odis.core.MessageProcessor;
import org.ow2.odis.core.ThreadManager;


/**
 * look on the serial port if a new message is arrived.
 * @author S.Bogaert FranceTelecom R&D
 */
public class MessageListener extends MessageProcessor implements
        IMessageListener, Runnable {
    /** logger of this class/. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            MessageListener.class.getName());

    /** The attribute. */
    SqlConnectionAttribute attribute;

    /** The read thread. */
    Thread readThread = null;

    /**
     * Instantiates a new message listener.
     * @param server the server
     * @param serialPort the serial port
     */
    MessageListener(AbstractConnectionIn server, IConnection cnx) {
        super(server, cnx);
        readThread = new Thread(this, "SQL.MessageListener");
        ThreadManager.getInstance().add(this);
    }

    /** if an error occurred stopListening is fixed to true. */
    public boolean stopListening = false;

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.RS232.MessageActor#start()
     */
    public void start() {
        readThread.start();
    }

    /*
     * (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run() {
        ((AbstractConnectionIn) server).attribute.incNumberConnexion(1);
        try {
            while (!stopProcessing) {
                try {
                    try {
                        List payloads = (((IProtocolIn) protocol).read());
                        for (Iterator iterator = payloads.iterator(); iterator
                                .hasNext();) {
                            byte[] payload = (byte[]) iterator.next();
                            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                                LOGGER.log(BasicLevel.DEBUG, "Add message");
                            }
                            ((AbstractConnectionIn) server).addMessage(payload,
                                    context);
                        }
                    } catch (Exception e) {
                        if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                            LOGGER.log(BasicLevel.ERROR, e.getMessage());
                        }
                    }
                    Thread.sleep(attribute.getInterval());
                } catch (InterruptedException e) {
                    if (LOGGER.isLoggable(BasicLevel.INFO)) {
                        LOGGER.log(BasicLevel.INFO,
                                "interrupt exception during read protocol");
                    }
                }
            }
        } catch (RuntimeException e) {
            LOGGER.log(BasicLevel.ERROR, "Listen exception :", e);
        }
        LOGGER.log(BasicLevel.INFO, "END LISTENING //" + hashCode());
        stopListening = true;
        readThread = null;
        ((AbstractConnectionIn) server).attribute.incNumberConnexion(-1);
    }

    /**
     * Gets the attribute.
     * @return the attribute
     */
    public SqlConnectionAttribute getAttribute() {
        return attribute;
    }

    /**
     * Sets the attribute.
     * @param attribut the new attribute
     */
    public void setAttribute(SqlConnectionAttribute attribute) {
        this.attribute = attribute;
    }

    /**
    * Stop
    */
    public void stop() {
	stopProcessing = true;
    }
}
