package org.ow2.odis.sql;

import java.math.BigDecimal;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisException;


/**
 * @author lqzp4386
 */
public class ResultSetHelper {
    private static String dateFormat;

    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            ResultSetHelper.class.getName());

    public static void setInStatement(PreparedStatement pstmt,
            int parameterIndex, String type, Object value) throws SQLException,
            OdisAttributeException {
        if ((type != null) && (value != null)) {
            if ("BigDecimal".toLowerCase().equals(type.toLowerCase())) {
                if ("".equals((String) value) || value == null) {
                    pstmt.setBigDecimal(parameterIndex, null);
                } else {
                    pstmt.setBigDecimal(parameterIndex, new BigDecimal(
                            (String) value));
                }
            } else if ("Blob".toLowerCase().equals(type.toLowerCase())) {
                throw new OdisAttributeException(
                        "java.sql.Blob type is not supported yet.");
            } else if ("Boolean".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setBoolean(parameterIndex, Boolean
                        .parseBoolean((String) value));
            } else if ("Byte".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setByte(parameterIndex, Byte.parseByte((String) value));
            } else if ("Bytes".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setBytes(parameterIndex, (byte[]) value);
            } else if ("Clob".toLowerCase().equals(type.toLowerCase())) {
                throw new OdisAttributeException(
                        "java.sql.Clob type is not supported yet.");
            } else if ("Date".toLowerCase().equals(type.toLowerCase())) {
                Date date = new java.sql.Date(getJavaDate((String) value)
                        .getTime());
                pstmt.setDate(parameterIndex, date);
            } else if ("Double".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setDouble(parameterIndex, Double
                        .parseDouble((String) value));
            } else if ("Float".toLowerCase().equals(type.toLowerCase())) {
                pstmt
                        .setFloat(parameterIndex, Float
                                .parseFloat((String) value));
            } else if ("Int".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setInt(parameterIndex, Integer.parseInt((String) value));
            } else if ("Integer".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setInt(parameterIndex, ((Integer) value).intValue());
            } else if ("Long".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setLong(parameterIndex, Long.parseLong((String) value));
            } else if ("Object".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setObject(parameterIndex, value);
            } else if ("Ref".toLowerCase().equals(type.toLowerCase())) {
                throw new OdisAttributeException(
                        "java.sql.Ref type is not supported yet.");
            } else if ("String".toLowerCase().equals(type.toLowerCase())) {
                pstmt.setString(parameterIndex, (String) value);
            } else if ("Short".toLowerCase().equals(type.toLowerCase())) {
                pstmt
                        .setShort(parameterIndex, Short
                                .parseShort((String) value));
            } else if ("Time".toLowerCase().equals(type.toLowerCase())) {
                Time time = new Time(getJavaDate((String) value).getTime());
                pstmt.setTime(parameterIndex, time);
            } else if ("Timestamp".toLowerCase().equals(type.toLowerCase())) {
                Timestamp timestamp = new Timestamp(getJavaDate((String) value)
                        .getTime());
                pstmt.setTimestamp(parameterIndex, timestamp);
            } else if ("URL".toLowerCase().equals(type.toLowerCase())) {
                try {
                    pstmt.setURL(parameterIndex, new URL((String) value));
                } catch (MalformedURLException e) {
                    throw new OdisAttributeException(e.getClass() + " "
                            + e.getMessage());
                }
            } else {
                throw new OdisAttributeException("The type " + type
                        + " is not reconized.");
            }
        }
    }

    protected static java.util.Date getJavaDate(String value)
            throws OdisAttributeException {
        java.util.Date result = null;
        if (value != null) {
            DateFormat formater = new SimpleDateFormat(dateFormat,
                    java.util.Locale.FRENCH);
            try {
                result = formater.parse((String) value);
            } catch (Exception e) {
                throw new OdisAttributeException(e.getMessage()
                        + " - date format needed: " + dateFormat
                        + " (value received = " + value + ")");
            }
        }
        return result;
    }

    /**
     * Gets the value.
     * @param rs the rs
     * @param columnName the column name
     * @param type the type
     * @return an array : the column name in the first column (String) + the object value in the second column (Object) + the java class
     * name in the third column (String)
     * @throws SQLException the SQL exception
     * @throws OdisException the odis exception
     */
    public static Object[] getValue(ResultSet rs, String columnName, String type)
            throws SQLException, OdisAttributeException {
        return getValue(rs, columnName, getSqlTypeValue(type));
    }

    /**
     * Gets the value.
     * @param rs the rs
     * @param row the row
     * @param columnIndex the column index
     * @param type the type
     * @return an array : the column name in the first column (String) + the object value in the second column (Object) + the java class
     * name in the third column (String)
     * @throws SQLException the SQL exception
     * @throws OdisException the odis exception
     */
    public static Object[] getValue(ResultSet rs, String columnName, int type)
            throws SQLException, OdisAttributeException {
        Object value = null;
        String className = null;
        switch (type) {
        case Types.ARRAY:
            value = rs.getArray(columnName);
            className = java.sql.Array.class.getName();
            break;
        case Types.BIGINT:
            value = Long.valueOf(rs.getLong(columnName));
            className = Long.class.getName();
            break;
        case Types.BINARY:
            value = rs.getBytes(columnName);
            className = byte.class.getName();
            break;
        case Types.BIT:
            value = Boolean.valueOf(rs.getBoolean(columnName));
            className = Boolean.class.getName();
            break;
        case Types.BLOB:
            value = rs.getBlob(columnName);
            className = Blob.class.getName();
            break;
        case Types.BOOLEAN:
            value = Boolean.valueOf(rs.getBoolean(columnName));
            className = Boolean.class.getName();
            break;
        case Types.CHAR:
            value = rs.getString(columnName);
            className = String.class.getName();
            break;
        case Types.CLOB:
            value = rs.getClob(columnName);
            className = Clob.class.getName();
            break;
        case Types.DATALINK:
            value = rs.getURL(columnName);
            className = URL.class.getName();
            break;
        case Types.DATE:
            value = rs.getDate(columnName);
            className = Date.class.getName();
            break;
        case Types.DECIMAL:
            value = rs.getBigDecimal(columnName);
            className = BigDecimal.class.getName();
            break;
        case Types.DISTINCT:
            throw new OdisAttributeException(
                    "The java.sql.type.DISTINCT type is not supported.");
        case Types.DOUBLE:
            value = Double.valueOf(rs.getDouble(columnName));
            className = Double.class.getName();
            break;
        case Types.FLOAT:
            value = Double.valueOf(rs.getDouble(columnName));
            className = Double.class.getName();
            break;
        case Types.INTEGER:
            value = Integer.valueOf(rs.getInt(columnName));
            className = Integer.class.getName();
            break;
        case Types.JAVA_OBJECT:
            value = rs.getObject(columnName);
            className = Object.class.getName();
            break;
        case Types.LONGVARBINARY:
            value = rs.getBytes(columnName);
            className = byte[].class.getName();
            break;
        case Types.NULL:
            throw new OdisAttributeException(
                    "The java.sql.type.NULL type is not supported.");
        case Types.NUMERIC:
            value = rs.getBigDecimal(columnName);
            className = BigDecimal.class.getName();
            break;
        case Types.OTHER:
            throw new OdisAttributeException(
                    "The java.sql.type.OTHER type is not supported.");
        case Types.REAL:
            value = Float.valueOf(rs.getFloat(columnName));
            className = Float.class.getName();
            break;
        case Types.REF:
            value = rs.getRef(columnName);
            className = Ref.class.getName();
            break;
        case Types.SMALLINT:
            value = Short.valueOf(rs.getShort(columnName));
            className = Short.class.getName();
            break;
        case Types.STRUCT:
            throw new OdisAttributeException(
                    "The java.sql.type.STRUCT type is not supported.");
        case Types.TIME:
            value = rs.getTime(columnName);
            className = Time.class.getName();
            break;
        case Types.TIMESTAMP:
            value = rs.getTimestamp(columnName);
            className = Timestamp.class.getName();
            break;
        case Types.TINYINT:
            value = Byte.valueOf(rs.getByte(columnName));
            className = Byte.class.getName();
            break;
        case Types.VARBINARY:
            value = rs.getBytes(columnName);
            className = byte[].class.getName();
            break;
        case Types.VARCHAR:
            value = rs.getString(columnName);
            className = String.class.getName();
            break;
        default:
            throw new OdisAttributeException("The type \"" + type
                    + "\" is not reconized as a java.sql.type.");
        }
        Object result[] = { columnName, value, className };
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "[" + columnName + ", " + value + ", "
                    + className + "]");
        }
        return result;
    }

    /**
     * @param typeName
     * @return
     * @throws OdisException
     */
    public static int getSqlTypeValue(String typeName)
            throws OdisAttributeException {
        int result;
        try {
            result = getSqlTypeValue_fromJavaType(typeName);
        } catch (OdisAttributeException e) {
            try {
                result = getSqlTypeValue_fromSqlType(typeName);
            } catch (OdisException e1) {
                throw e;
            }
        }
        return result;
    }

    /**
     * Gets the sql type value.
     * @param typeName the type name
     * @return the sql type value
     * @throws OdisException the odis exception
     */
    private static int getSqlTypeValue_fromJavaType(String typeName)
            throws OdisAttributeException {
        int result;
        if ("ARRAY".equals(typeName.toUpperCase()))
            result = Types.ARRAY;
        else if ("LONG".equals(typeName.toUpperCase()))
            result = Types.BIGINT;
        else if ("BYTES".equals(typeName.toUpperCase()))
            result = Types.BINARY;
        else if ("BOOLEAN".equals(typeName.toUpperCase()))
            result = Types.BIT;
        else if ("BLOB".equals(typeName.toUpperCase()))
            result = Types.BLOB;
        else if ("CLOB".equals(typeName.toUpperCase()))
            result = Types.CLOB;
        else if ("DATE".equals(typeName.toUpperCase()))
            result = Types.DATE;
        else if ("BIGDECIMAL".equals(typeName.toUpperCase()))
            result = Types.DECIMAL;
        else if ("DECIMAL".equals(typeName.toUpperCase()))
            result = Types.DECIMAL;
        else if ("DOUBLE".equals(typeName.toUpperCase()))
            result = Types.DOUBLE;
        else if ("INTEGER".equals(typeName.toUpperCase()))
            result = Types.INTEGER;
        else if ("INT".equals(typeName.toUpperCase()))
            result = Types.INTEGER;
        else if ("JAVA_OBJECT".equals(typeName.toUpperCase()))
            result = Types.JAVA_OBJECT;
        else if ("STRING".equals(typeName.toUpperCase()))
            result = Types.VARCHAR;
        else if ("FLOAT".equals(typeName.toUpperCase()))
            result = Types.REAL;
        else if ("REF".equals(typeName.toUpperCase()))
            result = Types.REF;
        else if ("SHORT".equals(typeName.toUpperCase()))
            result = Types.SMALLINT;
        else if ("TIME".equals(typeName.toUpperCase()))
            result = Types.TIME;
        else if ("TIMESTAMP".equals(typeName.toUpperCase()))
            result = Types.TIMESTAMP;
        else if ("BYTE".equals(typeName.toUpperCase()))
            result = Types.TINYINT;
        else
            throw new OdisAttributeException("The type \""
                    + typeName.toUpperCase()
                    + "\" is not reconized as a java.sql.type.");
        return result;
    }

    /**
     * Gets the sql type value.
     * @param typeName the type name
     * @return the sql type value
     * @throws OdisException the odis exception
     */
    private static int getSqlTypeValue_fromSqlType(String typeName)
            throws OdisException {
        int result;
        if ("ARRAY".equals(typeName.toUpperCase()))
            result = Types.ARRAY;
        else if ("BIGINT".equals(typeName.toUpperCase()))
            result = Types.BIGINT;
        else if ("BINARY".equals(typeName.toUpperCase()))
            result = Types.BINARY;
        else if ("BIT".equals(typeName.toUpperCase()))
            result = Types.BIT;
        else if ("BLOB".equals(typeName.toUpperCase()))
            result = Types.BLOB;
        else if ("BOOLEAN".equals(typeName.toUpperCase()))
            result = Types.BOOLEAN;
        else if ("CHAR".equals(typeName.toUpperCase()))
            result = Types.CHAR;
        else if ("CLOB".equals(typeName.toUpperCase()))
            result = Types.CLOB;
        else if ("DATALINK".equals(typeName.toUpperCase()))
            result = Types.DATALINK;
        else if ("DATE".equals(typeName.toUpperCase()))
            result = Types.DATE;
        else if ("DECIMAL".equals(typeName.toUpperCase()))
            result = Types.DECIMAL;
        else if ("DISTINCT".equals(typeName.toUpperCase()))
            result = Types.DISTINCT;
        else if ("DOUBLE".equals(typeName.toUpperCase()))
            result = Types.DOUBLE;
        else if ("FLOAT".equals(typeName.toUpperCase()))
            result = Types.FLOAT;
        else if ("INTEGER".equals(typeName.toUpperCase()))
            result = Types.INTEGER;
        else if ("INT".equals(typeName.toUpperCase()))
            result = Types.INTEGER;
        else if ("JAVA_OBJECT".equals(typeName.toUpperCase()))
            result = Types.JAVA_OBJECT;
        else if ("LONGVARBINARY".equals(typeName.toUpperCase()))
            result = Types.LONGVARBINARY;
        else if ("NULL".equals(typeName.toUpperCase()))
            result = Types.NULL;
        else if ("NUMERIC".equals(typeName.toUpperCase()))
            result = Types.NUMERIC;
        else if ("OTHER".equals(typeName.toUpperCase()))
            result = Types.OTHER;
        else if ("REAL".equals(typeName.toUpperCase()))
            result = Types.REAL;
        else if ("REF".equals(typeName.toUpperCase()))
            result = Types.REF;
        else if ("SMALLINT".equals(typeName.toUpperCase()))
            result = Types.SMALLINT;
        else if ("STRUCT".equals(typeName.toUpperCase()))
            result = Types.STRUCT;
        else if ("TIME".equals(typeName.toUpperCase()))
            result = Types.TIME;
        else if ("TIMESTAMP".equals(typeName.toUpperCase()))
            result = Types.TIMESTAMP;
        else if ("TINYINT".equals(typeName.toUpperCase()))
            result = Types.TINYINT;
        else if ("VARBINARY".equals(typeName.toUpperCase()))
            result = Types.VARBINARY;
        else if ("VARCHAR".equals(typeName.toUpperCase()))
            result = Types.VARCHAR;
        else
            throw new OdisAttributeException("The type \""
                    + typeName.toUpperCase()
                    + "\" is not reconized as a java.sql.type.");
        return result;
    }

    /**
     * Gets the date format
     * @return dateFormat
     */
    public static String getDateFormat() {
        return dateFormat;
    }

    /**
     * Sets the date format
     * @param dateFormat
     */
    public static void setDateFormat(String dateFormat) {
        ResultSetHelper.dateFormat = dateFormat;
    }
}
