package org.ow2.odis.sql;

import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jdom.Element;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.connection.context.ContextMessage;
import org.ow2.odis.core.AConnectionAttribute;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.model.AbstractAttribute;
import org.ow2.odis.model.Const;


/**
 * Defines the attribute for a serial port monodirectional connection.
 * @author lqzp4386
 */
public class SqlConnectionAttribute extends AConnectionAttribute implements SqlConnectionAttributeMBean {
    /**
     * Instantiates a new monodirectional cnx attribute.
     * @param parent the parent
     */
    public SqlConnectionAttribute(AbstractAttribute parent) {
        super(parent);
    }

    /** True if this is already initialized */
    private boolean isInitialized = false;

    /** The Constant PROPERTY_PROPERTIES. */
    static final String PROPERTY_PROPERTIES = "PROPERTIES";

    /** The Constant PROPERTY_IN. */
    public static final String PROPERTY_IN = "IN";

    /** The Constant PROPERTY_ACTION_QUERIES. */
    public static final String PROPERTY_ACTION_QUERIES = "ACTION_QUERIES";

    /** The Constant PROPERTY_URL. */
    public static final String PROPERTY_URL = "URL";

    /** The Constant PROPERTY_LOGIN. */
    public static final String PROPERTY_LOGIN = "LOGIN";

    /** The Constant PROPERTY_PASSWORD. */
    public static final String PROPERTY_PASSWORD = "PASSWORD";

    /** The Constant PROPERTY_DRIVER. */
    public static final String PROPERTY_DRIVER = "DRIVER";

    /** The Constant PROPERTY_ACTION_SELECT. */
    public static final String PROPERTY_ACTION_SELECT = "ACTION_SELECT";

    /** The Constant PROPERTY_INTERVAL. */
    public static final String PROPERTY_INTERVAL = "INTERVAL";

    /** The Constant PROPERTY_ACTION_ACKNOWLEDGMENT. */
    public static final String PROPERTY_ACTION_ACKNOWLEDGMENT = "ACTION_ACKNOWLEDGMENT";

    /** The Constant PROPERTY_ACTION_QUERY. */
    public static final String PROPERTY_ACTION_QUERY = "ACTION_QUERY";

    /** The Constant PROPERTY_POOL_SIZE. */
    public static final String PROPERTY_POOL_SIZE = "POOL_SIZE";

    /** The Constant PROPERTY_DATE_FORMAT. */
    public static final String PROPERTY_DATE_FORMAT = "DATE_FORMAT";

    /** The driver. */
    private String driver = "";

    /** The database. */
    private String database = "";

    /** The login. */
    private String login = "";

    /** The password. */
    private String password = "";

    /** The select query. */
    private String selectQueryName;

    /** The string interval between two select queries. */
    private String strInterval;

    /** The interval between two select queries. */
    private int interval = 1000;

    /** The ack query. */
    private String ackQueryName;

    /** The action queries. */
    private Map actionQueries = new HashMap();

    /** The action queries list. */
    private String[][] actionQueriesList;

    /** The url. */
    private String url = "";

    /** The default pool size. */
    private int defaultPoolSize = 5;

    /** The str pool size. */
    private String strPoolSize;

    /** The pool size. */
    private int poolSize = defaultPoolSize;

    /** The default date format. */
    private String defaulDateFormat = "yyyy/MM/dd HH.mm.ss";

    /** The date format. */
    private String dateFormat = defaulDateFormat;

    /**
     * equals.
     * @param obj to be compare
     * @return boolean
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        boolean result = false;
        SqlConnectionAttribute copy = null;
        try {
            copy = (SqlConnectionAttribute ) obj;
        } catch (ClassCastException e) {
            return false;
        }
        result = super.equals(obj);
        if (result) {
            result &= ((driver == null && copy.driver == null) || copy.driver
                    .equals(this.driver));
            result &= ((database == null && copy.database == null) || copy.database
                    .equals(this.database));
            result &= ((url == null && copy.url == null) || copy.url
                    .equals(this.url));
            result &= ((login == null && copy.login == null) || copy.login
                    .equals(this.login));
            result &= ((password == null && copy.password == null) || copy.password
                    .equals(this.password));
            result &= ((strInterval == null && copy.strInterval == null) || copy.strInterval
                    .equals(this.strInterval));
            result &= ((selectQueryName == null && copy.selectQueryName == null) || copy.selectQueryName
                    .equals(this.selectQueryName));
            result &= ((ackQueryName == null && copy.ackQueryName == null) || copy.ackQueryName
                    .equals(this.ackQueryName));
            result &= ((dateFormat == null && copy.dateFormat == null) || copy.dateFormat
                    .equals(this.dateFormat));
            result &= ((getActionQueriesAsString() == null && copy
                    .getActionQueriesAsString() == null) || copy
                    .getActionQueriesAsString().equals(
                            this.getActionQueriesAsString()));
            if (!result) {
                traceOnConfictualConnection(copy);
            }
        }
        return result;
    }

    /**
     * Setter.
     * @param declaration the declaration
     */
    public void subSetter(Element declaration) {
        List lsUnusedProperties = declaration.getChildren();
        Element element = declaration.getChild(PROPERTY_DRIVER);
        if (element != null) {
            driver = element.getAttributeValue(Const.VALUE, driver);
            lsUnusedProperties.remove(element);
        }
        element = declaration.getChild(PROPERTY_LOGIN);
        if (element != null) {
            login = element.getAttributeValue(Const.VALUE, login);
            lsUnusedProperties.remove(element);
        }
        element = declaration.getChild(PROPERTY_PASSWORD);
        if (element != null) {
            password = element.getAttributeValue(Const.VALUE, password);
            lsUnusedProperties.remove(element);
        }
        element = declaration.getChild(PROPERTY_URL);
        if (element != null) {
            url = element.getAttributeValue(Const.VALUE, url);
            lsUnusedProperties.remove(element);
        }
        element = declaration.getChild(PROPERTY_IN);
        if (element != null) {
            Element subElement = (Element) element.getChild(PROPERTY_INTERVAL);
            if (subElement != null) {
                strInterval = subElement.getAttributeValue(Const.VALUE,
                        strInterval);
            }
            subElement = (Element) element.getChild(PROPERTY_ACTION_SELECT);
            if (subElement != null) {
                selectQueryName = subElement.getAttributeValue(Const.VALUE,
                        selectQueryName);
            }
            subElement = (Element) element
                    .getChild(PROPERTY_ACTION_ACKNOWLEDGMENT);
            if (subElement != null) {
                ackQueryName = subElement.getAttributeValue(Const.VALUE,
                        ackQueryName);
            }
            lsUnusedProperties.remove(element);
        }
        element = declaration.getChild(PROPERTY_ACTION_QUERIES);
        if (element != null) {
            List listElement = element.getChildren(PROPERTY_ACTION_QUERY);
            Iterator i = listElement.iterator();
            int nbAction = 0;
            actionQueriesList = new String[listElement.size()][2];
            while (i.hasNext()) {
                Element subElement = (Element) i.next();
                String[] query = { subElement.getAttributeValue("name", ""),
                        subElement.getText() };
                actionQueriesList[nbAction++] = query;
            }
            lsUnusedProperties.remove(element);
        }
        element = declaration.getChild(PROPERTY_POOL_SIZE);
        if (element != null) {
            strPoolSize = element.getAttributeValue(Const.VALUE, strPoolSize);
            lsUnusedProperties.remove(element);
        }
        element = declaration.getChild(PROPERTY_DATE_FORMAT);
        if (element != null) {
            dateFormat = element.getAttributeValue(Const.VALUE, dateFormat);
            lsUnusedProperties.remove(element);
        }
        super.subSetter(declaration);
    }

    /**
     * getter.
     * @return the type of this connection.
     */
    public String getTypeConnection() {
        return "SQL";
    }

    /**
     * Checks the validity of all SqlCnxAttribut parameters.
     * @throws OdisAttributException exception in case of error
     * @see com.francetelecom.odis.model.AbstractAttribut#checkProperties()
     */
    public void checkProperties(StringBuffer buff) {
        if (!isInitialized) {
            if ("".equals(driver)) {
                buff.append(PROPERTY_DRIVER + " is mandatory.").append(
                        Const.EOL);
            }
            if ("".equals(this.login)) {
                buff.append(PROPERTY_LOGIN + " is mandatory.")
                        .append(Const.EOL);
            }
            if ("".equals(this.url)) {
                buff.append(PROPERTY_URL + " is mandatory.").append(Const.EOL);
            }
            if ("".equals(this.dateFormat)) {
                buff.append(PROPERTY_DATE_FORMAT + " is mandatory.").append(
                        Const.EOL);
            } else {
                ResultSetHelper.setDateFormat(dateFormat);
            }
            if ((this.strPoolSize != null) && (!"".equals(this.strPoolSize))) {
                try {
                    poolSize = Integer.parseInt(strPoolSize);
                } catch (NumberFormatException e) {
                    buff.append(PROPERTY_POOL_SIZE + " must be an integer.")
                            .append(Const.EOL);
                }
            }
            if ((this.strInterval != null) && (!"".equals(this.strInterval))) {
                try {
                    interval = Integer.parseInt(strInterval);
                } catch (NumberFormatException e) {
                    buff.append(PROPERTY_INTERVAL + " must be an integer.")
                            .append(Const.EOL);
                }
            }
            if (this.actionQueriesList != null) {
                try {
                    setActionQueries(actionQueriesList);
                } catch (OdisAttributeException e) {
                    buff.append(e.getMessage()).append(Const.EOL);
                }
            }
            isInitialized = true;
        }
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getActionQueries()
     */
    public String getActionQueriesAsString() {
        StringBuffer buffer = new StringBuffer();
        Iterator iterator = actionQueries.keySet().iterator();
        while (iterator.hasNext()) {
            if ("".equals(buffer.toString())) {
                buffer.append("[");
            } else {
                buffer.append(", ");
            }
            String name = (String) iterator.next();
            String query = "";
            try {
                query = getQuery(name);
            } catch (OdisAttributeException e) {
                if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                    LOGGER.log(BasicLevel.DEBUG, e.getClass() + " "
                            + e.getMessage());
                }
            }
            buffer.append("[").append(name).append("] ").append(query);
        }
        buffer.append("]");
        return buffer.toString();
    }

    public Map getActionQueries() {
        return this.actionQueries;
    }

    public String getQuery(String name) throws OdisAttributeException {
        String result = null;
        result = (String) this.actionQueries.get(name);
        if (result == null || "".equals(result))
            throw new OdisAttributeException("No query named \"" + name
                    + "\" found.");
        return result;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getDriver()
     */
    public String getDriver() {
        return driver;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getDatabase()
     */
    public String getDatabase() {
        return database;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getLogin()
     */
    public String getLogin() {
        return login;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getPassword()
     */
    public String getPassword() {
        return password;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getSelectQuery()
     */
    public String getSelectQueryName() {
        return selectQueryName;
    }

    public String getAckQueryName() {
        return ackQueryName;
    }

    public void setAckQueryName(String ackQueryName) {
        this.ackQueryName = ackQueryName;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getUrl()
     */
    public String getUrl() {
        return url;
    }

    /**
     * Write XML.
     * @param out the out
     * @param full the full
     * @throws java.io.IOException *
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public void writeXML(OutputStream out, boolean full) throws IOException {
        out.write((Const.TAB + "<SQL name=\"" + getName() + "\">" + Const.EOL)
                .getBytes());
        super.writeXML(out, true);
        writeXMLCNXUtilities(out, 2, PROPERTY_URL, getUrl(), "", full);
        writeXMLCNXUtilities(out, 2, PROPERTY_LOGIN, getLogin(), "", full);
        writeXMLCNXUtilities(out, 2, PROPERTY_PASSWORD, getPassword(), "", full);
        writeXMLCNXUtilities(out, 2, PROPERTY_DRIVER, getDriver(), "", full);
        writeXMLCNXUtilities(out, 2, PROPERTY_POOL_SIZE, getPoolSize(), Integer
                .toString(defaultPoolSize), full);
        writeXMLCNXUtilities(out, 2, PROPERTY_DATE_FORMAT, getDateFormat(),
                defaulDateFormat, full);
        if ((getSelectQueryName() != null)
                && (!"".equals(getSelectQueryName()))) {
            writeXMLCNXUtilities(out, 2, PROPERTY_IN, false);
            writeXMLCNXUtilities(out, 3, PROPERTY_INTERVAL, Integer
                    .toString(getInterval()), "", full);
            writeXMLCNXUtilities(out, 3, PROPERTY_ACTION_SELECT,
                    getSelectQueryName(), "", full);
            writeXMLCNXUtilities(out, 3, PROPERTY_ACTION_ACKNOWLEDGMENT,
                    getAckQueryName(), "", full);
            writeXMLCNXUtilities(out, 2, PROPERTY_IN, true);
        }
        if (actionQueries != null && actionQueries.size() != 0) {
            writeXMLCNXUtilities(out, 2, PROPERTY_ACTION_QUERIES, false);
            Iterator iterator = actionQueries.keySet().iterator();
            while (iterator.hasNext()) {
                String name = (String) iterator.next();
                try {
                    writeXMLCNXUtilities(out, 3, PROPERTY_ACTION_QUERY, "name",
                            name, getQuery(name));
                } catch (OdisAttributeException e) {
                    if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                        LOGGER.log(BasicLevel.ERROR, e.getClass() + " "
                                + e.getMessage());
                    }
                }
            }
            writeXMLCNXUtilities(out, 2, PROPERTY_ACTION_QUERIES, true);
        }
        out.write((Const.TAB + "</SQL>" + Const.EOL).getBytes());
    }

    /**
     * Gets the P name.
     * @return the p name
     * @see com.francetelecom.odis.model.AbstractAttribut#getPName()
     */
    public String getPName() {
        StringBuffer buffer = new StringBuffer(getParent().getPName());
        buffer.append(Const.CST_SEP);
        buffer.append("SqlCnx");
        return buffer.toString();
    }

    /**
     * Sets the driver.
     * @param driver the new driver
     */
    public void setDriver(String driver) {
        this.driver = driver;
    }

    /**
     * Sets the database.
     * @param database the new database
     */
    public void setDatabase(String database) {
        this.database = database;
    }

    /**
     * Sets the login.
     * @param login the new login
     */
    public void setLogin(String login) {
        this.login = login;
    }

    /**
     * Sets the password.
     * @param password the new password
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * Sets the pool size.
     * @param poolSize the pool size
     */
    public void setPoolSize(int poolSize) {
        this.poolSize = poolSize;
    }

    /**
     * Sets the select query.
     * @param selectQueryName the new select query
     */
    public void setSelectQueryName(String selectQueryName) {
        this.selectQueryName = selectQueryName;
    }

    /**
     * Sets the action queries.
     * @param actionQueries the new action queries
     */
    public void setActionQueries(HashMap actionQueries) {
        this.actionQueries = actionQueries;
    }

    /**
     * Sets the action queries.
     * @param actionQueries the new action queries
     */
    public void setActionQueries(String[][] actionQueries)
            throws OdisAttributeException {
        for (int i = 0; i < actionQueries.length; i++) {
            addActionQuery(actionQueries[i][0], actionQueries[i][1]);
        }
    }

    /**
     * Sets the url.
     * @param url the new url
     */
    public void setUrl(String url) {
        this.url = url;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#addOrModifyActionQuery(java.lang.String)
     */
    public boolean addOrModifyActionQuery(String name, String query) {
        boolean result = true;
        try {
            if (actionQueries.containsKey(name)) {
                modifyActionQuery(name, query);
            } else {
                addActionQuery(name, query);
            }
        } catch (OdisAttributeException e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, e.getClass() + " "
                        + e.getMessage());
            }
            result = false;
        }
        return result;
    }

    /**
     * Modify action query.
     * @param name the name
     * @param query the query
     * @throws OdisAttributException the odis attribut exception
     */
    protected void modifyActionQuery(String name, String query)
            throws OdisAttributeException {
        if (!actionQueries.containsKey(name)) {
            addActionQuery(name, query);
        } else {
            if (isEmpty(query))
                throw new OdisAttributeException(
                        "The query of the action query is mandatory.");
            actionQueries.remove(name);
            actionQueries.put(name, query);
        }
    }

    /**
     * Adds the action query.
     * @param name the name
     * @param query the query
     * @throws OdisAttributException the odis attribut exception
     */
    protected void addActionQuery(String name, String query)
            throws OdisAttributeException {
        if (actionQueries.containsKey(name))
            throw new OdisAttributeException("An ACTION_QUERY with the name \""
                    + name + "\" already exists.");
        if (isEmpty(name))
            throw new OdisAttributeException(
                    "The name of the ACTION_QUERY is mandatory.");
        if (isEmpty(query))
            throw new OdisAttributeException(
                    "The query of the ACTION_QUERY is mandatory.");
        actionQueries.put(name, query);
    }

    private boolean isEmpty(String string) {
        return ((string == null) || ("".equals(string)));
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#removeActionQuery(java.lang.String)
     */
    public boolean removeActionQuery(String name) {
        // TODO Auto-generated method stub
        return false;
    }

    /**
     * Gets the query string.
     * @param msg the msg
     * @return the query string
     */
    public String getActionQueryString(ContextMessage msg)
            throws OdisAttributeException {
        String result = null;
        if ((msg != null) && (msg.getContext() != null)
                && (msg.getContext().getIdentity() != null)
                && (getActionQueries() != null)) {
            String actionName = msg.getContext().getIdentity();
            result = getQuery(actionName);
        } else {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER
                        .log(
                                BasicLevel.ERROR,
                                "To run action query, you need a valid contextual message and a listing of possible action queries.");
            }
        }
        return result;
    }

    /**
     * Return the StatementParameterBean array corresponding tho the context message
     * @param msg the ContextMaessage
     * @return a StatementParameterBean array
     * @throws OdisAttributException
     */
    public StatementParameterBean[] getParameters(ContextMessage msg)
            throws OdisAttributeException {
        StatementParameterBean[] result = null;
        if ((msg != null) && (msg.getContext() != null)
                && (msg.getContext().getSubIdentity() != null)
                && (msg.getContext().getDetail() != null)) {
            String[] values = msg.getContext().getSubIdentity().split(",");
            String[] types = msg.getContext().getDetail().split(",");
            if (values.length != types.length) {
                throw new OdisAttributeException(
                        "You must set the same quantity of values (in the ContextMessage subIdentity) than types (in the ContextMessage detail)");
            }
            result = new StatementParameterBean[values.length];
            for (int i = 0; i < types.length; i++) {
                StatementParameterBean parameter = new StatementParameterBean();
                parameter.setType(types[i].trim());
                String value = (String) values[i].trim();
                if ("null".equals(value.toLowerCase())) {
                    value = null;
                } else {
                    if ("PAYLOAD".equals(value)) {
                        parameter.setPayload(msg.getPayload());
                    } else {
                        parameter.setValue(value);
                    }
                }
                result[i] = parameter;
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.SqlCnxAttributMBean#getPoolSize()
     */
    public String getPoolSize() {
        return Integer.toString(poolSize);
    }

    /**
     * Gets the interval
     * @return interval
     */
    public int getInterval() {
        return interval;
    }

    /**
     * Gets the action queries list
     * @param actionQueriesList
     */
    public void setActionQueriesList(String[][] actionQueriesList) {
        this.actionQueriesList = actionQueriesList;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.ACnxAttribut#setDefaultProtocolIn()
     */
    protected void setDefaultProtocolIn() {
        super.setDefaultProtocolIn(SqlDefaultProtocol.class.getName());
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.ACnxAttribut#setDefaultProtocolOut()
     */
    protected void setDefaultProtocolOut() {
        super.setDefaultProtocolOut(SqlDefaultProtocol.class.getName());
    }

    /**
     * Gets the date format
     * @return dateFormat
     */
    public String getDateFormat() {
        return dateFormat;
    }

    /**
     * date format setter
     * @param dateFormat
     */
    public void setDateFormat(String dateFormat) {
        this.dateFormat = dateFormat;
    }
}
