package org.ow2.odis.sql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Vector;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisException;


/**
 * The Class SqlConnectionHandler.
 */
public class SqlConnectionHandler {
    /** The pool. */
    protected static Vector pool;

    /** The state. */
    protected static Vector state;

    /** The attribute. */
    protected static SqlConnectionAttribute attribute;

    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            SqlConnectionHandler.class.getName());

    /**
     * Gets the connection.
     * @return the connection
     */
    public static synchronized Connection getConnection() throws OdisException {
        Connection result = null;
        if (state == null || pool == null) {
            throw new OdisException(
                    "The connections pool is not properly initialized");
        }
        for (int i = 0; i < state.size(); i++) {
            if (state.get(i).equals("FREE")) {
                state.set(i, "USED");
                result = (Connection) pool.get(i);
                try {
                    if (result.isClosed()) {
                        if (LOGGER.isLoggable(BasicLevel.WARN)) {
                            LOGGER
                                    .log(BasicLevel.WARN,
                                            "The sql connection is closed: build a new one");
                        }
                        result = getSqlConnection();
                        pool.remove(i);
                        pool.add(result);
                    }
                } catch (Exception e) {
                    if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                        LOGGER.log(BasicLevel.ERROR, e.getClass() + " "
                                + e.getMessage());
                    }
                }
                break;
            }
        }
        if (result == null) {
            if (LOGGER.isLoggable(BasicLevel.WARN)) {
                LOGGER
                        .log(BasicLevel.WARN,
                                "none of the sql connection are free: add a new one in the pool");
            }
            try {
                result = addConnection("USED");
            } catch (ClassNotFoundException e) {
                if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                    LOGGER.log(BasicLevel.ERROR, "ClassNotFoundExecption: "
                            + e.getMessage());
                }
            } catch (SQLException e) {
                if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                    LOGGER.log(BasicLevel.ERROR, "SQLException: "
                            + e.getMessage());
                }
            }
        }
        return result;
    }

    /**
     * Gets the sql connection.
     * @return the sql connection
     * @throws SQLException the SQL exception
     * @throws ClassNotFoundException the class not found exception
     */
    protected static Connection getSqlConnection() throws SQLException,
            ClassNotFoundException {
        Connection connection = null;
        Class.forName(attribute.getDriver());
        connection = DriverManager.getConnection(attribute.getUrl(), attribute
                .getLogin(), attribute.getPassword());
        return connection;
    }

    private static Connection addConnection(String state) throws SQLException,
            ClassNotFoundException {
        Connection connection = getSqlConnection();
        SqlConnectionHandler.pool.add(connection);
        SqlConnectionHandler.state.add(state);
        return connection;
    }

    /**
     * Libère une connection en la restituant au pool de connection et en la marquant comme 'free'.
     * @param connection connection à libérer
     * @throws Exception si une erreur intervient pendant la restitution de la connection
     */
    public static void freeConnection(Connection connection) throws Exception {
        int index = pool.indexOf(connection);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "Libération de la connection "
                    + index);
        }
        state.set(index, "FREE");
    }

    /**
     * Initialise le pool de connections en créant le nombre de connections indiqué dans la variable POOL_SIZE et en les stockant dans
     * une liste. Toutes ces connections sont alors marquées comme 'free'
     */
    public static void init(SqlConnectionAttribute attribute) {
        SqlConnectionHandler.attribute = attribute;
        SqlConnectionHandler.pool = new Vector();
        SqlConnectionHandler.state = new Vector();
        try {
            // Le pool de connexion est créé uniquement si le paramètre POOL
            // est à true dans le fichier de configuration.
            for (int i = 0; i < Integer.parseInt(attribute.getPoolSize()); i++) {
                addConnection("FREE");
            }
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG,
                        "Pool de connections SQL créé (nombre de connexions : "
                                + attribute.getPoolSize() + ")");
            }
        } catch (ClassNotFoundException e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, "ClassNotFoundExecption: "
                        + e.getMessage());
            }
        } catch (SQLException e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, "SQLException: " + e.getMessage());
            }
        }
    }

    /**
     * Close.
     */
    public static void close() {
        for (int i = 0; i < SqlConnectionHandler.pool.size(); i++) {
            Connection conn = (Connection) SqlConnectionHandler.pool.get(i);
            try {
                conn.close();
            } catch (SQLException e) {
                if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                    LOGGER.log(BasicLevel.ERROR, "SQLException: "
                            + e.getMessage());
                }
            }
        }
    }
}
