package org.ow2.odis.sql;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.context.ContextMessage;
import org.ow2.odis.encoder.GenericEncoder;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisException;
import org.ow2.odis.model.AbstractConnectionAttribute;

/**
 * The Class SqlDefaultProtocol.
 */
public class SqlDefaultProtocol implements IProtocolIn, IProtocolOut {
    /** logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            SqlDefaultProtocol.class.getName());

    /** The attribute. */
    private SqlConnectionAttribute attribute;

    private String ID_COLUMN_NAME = "id";

    private String ID_COLUMN_TYPE = "Integer";

    /** The column index constant. */
    private final int COLUMN_NAME = 0;

    private final int COLUMN_INDEX = 1;

    private final int COLUMN_TYPE = 2;

    private String properties = null;

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.sql.IProtocolIn#read()
     */
    public List read() throws OdisAttributeException, SQLException {
        List result = new ArrayList();
        ResultSet rs = null;
        String selectQueryName = attribute.getSelectQueryName();
        if (selectQueryName == null || "".equals(selectQueryName))
            throw new OdisAttributeException(
                    "The select query name is not properly set.");
        rs = SqlConnectionHelper.executeQuery(attribute
                .getQuery(selectQueryName));
        Map payloads = getPayloads(rs);
        Set payloadsId = payloads.keySet();
        for (Iterator iterator = payloadsId.iterator(); iterator.hasNext();) {
            // add payload
            Object id = iterator.next();
            result.add(getByte((List) payloads.get(id)));
            // ack
            StatementParameterBean parameter = new StatementParameterBean();
            parameter.setType(ID_COLUMN_TYPE);
            parameter.setValue(id);
            StatementParameterBean[] parameters = { parameter };
            String ackQueryName = attribute.getAckQueryName();
            if (ackQueryName == null || "".equals(ackQueryName))
                throw new OdisAttributeException(
                        "The acknowledgment query name is not properly set.");
            SqlConnectionHelper.execute(attribute.getQuery(ackQueryName),
                    parameters);
        }
        //1 mn
        SqlConnectionHelper.closeAllOpenedPreparedStatement(1*60*10000);
        return result;
    }

    /*
     * (non-Javadoc)
     */
    public boolean write(ContextMessage message) {
        boolean result = true;
        try {
            String query = ((SqlConnectionAttribute) attribute)
                    .getActionQueryString(message);
            if ((query != null) || ("".equals(query))) {
                SqlConnectionHelper.execute(query, ((SqlConnectionAttribute) attribute)
                        .getParameters(message));
            } else {
                if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                    LOGGER
                            .log(BasicLevel.ERROR,
                                    "No sql query found, set the contextual message properly !");
                }
            }
        } catch (Exception e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, "Exception : ", e);
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.IProtocol#isStreamStillOpen()
     */
    public boolean isStreamStillOpen() {
        return false;
    }

    /**
     * Sets the abstract cnx attribut.
     * @param attribut the new abstract cnx attribut
     */
    public void setAbstractConnectionAttribute(AbstractConnectionAttribute attribute) {
        try {
            this.attribute = (SqlConnectionAttribute) attribute;
            if (attribute == null) {
                throw new RuntimeException("setAbstractCnxAttribut with null");
            }
        } catch (Exception e) {
            LOGGER.log(BasicLevel.ERROR, "setAbstractCnxAttribut with null", e);
        }
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.IProtocol#setProperties(java.lang.String)
     */
    public void setProperties(String properties) throws Exception {
        this.properties = properties;
        processProperties();
    }

    /**
     * example of properties : "-id ID -idType String"
     */
    private void processProperties() throws Exception {
        List listProperties;
        if (!"".equals(properties)) {
            listProperties = new ArrayList();
            StringTokenizer st = new StringTokenizer(properties, " ");
            while (st.hasMoreElements()) {
                listProperties.add(st.nextToken());
            }
            if (!((listProperties.size() == 2) || (listProperties.size() == 4))) {
                throw new Exception(
                        "Error while processing  protocol properties (example of properties : \"-id ID -idType int\")");
            }
            processToken(listProperties, 0);
            processToken(listProperties, 2);
        }
    }

    /**
     * Process token.
     * @param list the list
     * @param index the index
     * @throws Exception the exception
     */
    private void processToken(List list, int index) throws Exception {
        try {
            if ("-id".equals(list.get(index))) {
                setID_COLUMN_NAME((String) list.get(index + 1));
            } else if ("-idType".equals(list.get(index))) {
                setID_COLUMN_TYPE((String) list.get(index + 1));
            } else {
                throw new Exception(
                        "Error while processing  protocol properties (example of properties : \"-mode ACK -ack ok\")");
            }
        } catch (IndexOutOfBoundsException e) {
        }
    }

    public void setConnectionAttribute(AbstractConnectionAttribute attribute) {
        try {
            this.attribute = (SqlConnectionAttribute) attribute;
            if (attribute == null) {
                throw new RuntimeException("setConnectionAttribute with null");
            }
        } catch (Exception e) {
            LOGGER.log(BasicLevel.ERROR, "setConnectionAttribute with null", e);
        }
    }

    /**
     * Gets the byte by using GenericEncoder.
     * @param payload_values the payload_values
     * @return the byte
     */
    protected byte[] getByte(List payload_values) {
        GenericEncoder encoder = new GenericEncoder();
        return encoder.encode(payload_values);
    }

    /**
     * Return a Map of column value array (Map < Object id, List [String columnName, Object columnValue] >). The key corresponds to the id
     * of a row. This map represents all the <column name, column value> of a recordset rows
     * @param rs the rs
     * @return the payloads
     * @throws SQLException the SQL exception
     * @throws OdisException the odis exception
     */
    protected Map getPayloads(ResultSet rs) throws SQLException,
            OdisAttributeException {
        Map result = new HashMap();
        if (rs != null) {
            List columnProperties = getColumnProperties(rs);
            while (rs.next()) {
                List rowResult = new ArrayList();
                for (Iterator iterator = columnProperties.iterator(); iterator
                        .hasNext();) {
                    Object[] columnProperty = (Object[]) iterator.next();
                    Object[] o = ResultSetHelper.getValue(rs,
                            ((String) columnProperty[COLUMN_NAME]),
                            ((Integer) columnProperty[COLUMN_TYPE]).intValue());
                    rowResult.add(o);
                }
                Object[] idDescription = ResultSetHelper.getValue(rs,
                        ID_COLUMN_NAME, ID_COLUMN_TYPE);
                Object idValue = idDescription[1];
                result.put(idValue, rowResult);
            }
        }
        return result;
    }

    /**
     * First column: the column name Second column: the index Third column: the SQL type.
     * @param rs the rs
     * @return the column properties
     * @throws SQLException the SQL exception
     */
    private List getColumnProperties(ResultSet rs) throws SQLException {
        ResultSetMetaData metadata = rs.getMetaData();
        List result = new ArrayList();
        for (int i = 1; i < metadata.getColumnCount(); i++) {
            String name = metadata.getColumnName(i);
            Integer typeSQL = Integer.valueOf(metadata.getColumnType(i));
            Integer columnIndex = Integer.valueOf(i);
            // String nomTypeSQL = metadata.getColumnTypeName(i);
            // String typeJava = metadata.getColumnClassName(i);
            /*
             * if (LOGGER.isLoggable(BasicLevel.DEBUG)) { LOGGER.log(BasicLevel.DEBUG, "INFORMATIONS SUR LA COLONNE D'INDEXE
             * " + i); LOGGER.log(BasicLevel.DEBUG, "Nom : " + name); LOGGER.log(BasicLevel.DEBUG, "Column index : " + columnIndex);
             * LOGGER.log(BasicLevel.DEBUG, "Type SQL dans java.sql.Types : " + typeSQL); LOGGER.log(BasicLevel.DEBUG, "Nom du type SQL : "
             * + nomTypeSQL); LOGGER.log(BasicLevel.DEBUG, "Classe java correspondante : " + typeJava); LOGGER.log(BasicLevel.DEBUG,
             * "----------------------------------------------------"); }
             */
            Object property[] = { name, columnIndex, typeSQL };
            result.add(property);
        }
        return result;
    }

    /**
     * Gets the id column name.
     * @return the id column name
     */
    public String getID_COLUMN_NAME() {
        return ID_COLUMN_NAME;
    }

    /**
     * Gets the id column type.
     * @return the id column type
     */
    public String getID_COLUMN_TYPE() {
        return ID_COLUMN_TYPE;
    }

    /**
     * Sets the id column name.
     * @param id_column_name the new id column name
     */
    public void setID_COLUMN_NAME(String id_column_name) {
        ID_COLUMN_NAME = id_column_name;
    }

    /**
     * Sets the id column type.
     * @param id_column_type the new id column type
     */
    public void setID_COLUMN_TYPE(String id_column_type) {
        ID_COLUMN_TYPE = id_column_type;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.IProtocol#getProperties()
     */
    public String getProperties() {
        return properties;
    }
}
