package org.ow2.odis.sql;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.core.AbstractConnectionIn;
import org.ow2.odis.core.AbstractKeepConnexion;
import org.ow2.odis.core.IConnection;
import org.ow2.odis.core.IProtocol;
import org.ow2.odis.core.MessageProcessor;
import org.ow2.odis.exception.OdisAttributeException;


/**
 * The listener interface for receiving sqlKeepConnection events. The class that is interested in processing a sqlKeepConnection event
 * implements this interface, and the object created with that class is registered with a component using the component's
 * <code>addSqlKeepConnectionListener<code> method. When
 * the sqlKeepConnection event occurs, that object's appropriate
 * method is invoked.
 * @see SqlKeepConnectionEvent
 */
public class SqlKeepConnectionListener extends AbstractKeepConnexion {
    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            SqlKeepConnectionSender.class.getName());

    /**
     * Instantiates a new sql keep connection listener.
     * @param connexion the connexion
     * @param attribut the attribut
     */
    public SqlKeepConnectionListener(AbstractConnectionIn connexion,
    		SqlConnectionAttribute attribut) {
        super(connexion, attribut);
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.AbstractKeepConnexion#getMessageProcessor(com.francetelecom.odis.core.IConnection)
     */
    public MessageProcessor getMessageProcessor(IConnection cnx) {
        MessageListener listener;
        listener = new MessageListener((AbstractConnectionIn) connection, cnx);
        listener.setAttribute((SqlConnectionAttribute) this.attribute);
        return listener;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.AbstractKeepConnexion#getProtocol()
     */
    protected IProtocol getProtocol() {
        IProtocolOut result = null;
        try {
            result = ((SqlConnectionAttribute) attribute).createProtocolOutInstance();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "protocol is created");
            }
        } catch (OdisAttributeException e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, e.getMessage());
            }
        }
        return (IProtocol) result;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.AbstractKeepConnexion#getConnection()
     */
    protected IConnection getConnection() {
        return new SqlConnection();
    }
}
