package org.ow2.odis.sql;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.core.AbstractConnectionOut;
import org.ow2.odis.core.AbstractKeepConnexion;
import org.ow2.odis.core.IConnection;
import org.ow2.odis.core.IProtocol;
import org.ow2.odis.core.MessageProcessor;
import org.ow2.odis.exception.OdisAttributeException;


public class SqlKeepConnectionSender extends AbstractKeepConnexion {
    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            SqlKeepConnectionSender.class.getName());

    /**
     * Instantiates a new rXTX keep connexion out.
     * @param connexion the connexion
     * @param attribut the attribut
     */
    public SqlKeepConnectionSender(AbstractConnectionOut connexion,
    		SqlConnectionAttribute attribut) {
        super(connexion, attribut);
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.serialPort.RXTXKeepConnexion#getMessageProcessor(gnu.io.SerialPort)
     */
    public MessageProcessor getMessageProcessor(IConnection cnx) {
        MessageSender sender;
        sender = new MessageSender((AbstractConnectionOut) connection, cnx);
        sender.setAttribut((SqlConnectionAttribute) this.attribute);
        return sender;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.serialPort.RXTXKeepConnexion#getProtocol()
     */
    protected IProtocol getProtocol() {
        IProtocolOut result = null;
        try {
            result = ((SqlConnectionAttribute) attribute).createProtocolOutInstance();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "protocol is created");
            }
        } catch (OdisAttributeException e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, e.getMessage());
            }
        }
        return (IProtocol) result;
    }

    /*
     * (non-Javadoc)
     * @see com.francetelecom.odis.core.AbstractKeepConnexion#getConnection()
     */
    protected IConnection getConnection() {
        return new SqlConnection();
    }
}
